package co.ab180.airbridge.flutter

import android.content.Intent
import android.net.Uri
import co.ab180.airbridge.Airbridge
import co.ab180.airbridge.AirbridgeCallback
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.EventChannel
import java.util.*
import kotlin.collections.HashMap

class DeeplinkAPI(private val channel: EventChannel) : EventChannel.StreamHandler {
    companion object {
        val deeplinkTracker = DeeplinkTracker()
    }

    private var deeplinkListenerID: UUID? = null

    fun attachToChannel() {
        channel.setStreamHandler(this)
    }

    fun detachFromChannel() {
        channel.setStreamHandler(null)
    }

    override fun onListen(arguments: Any?, events: EventChannel.EventSink?) {
        val target = arguments as? String

        if (target == "deeplink") {
            events?.let {
                deeplinkListenerID = deeplinkTracker.listenDeeplink { deeplink ->
                    events.success(deeplink.toString())
                }
            }
        }
    }

    override fun onCancel(arguments: Any?) {
        deeplinkListenerID?.let { id ->
            deeplinkTracker.stopListenDeeplink(id)
        }
    }
}

class DeeplinkTracker {
    private var startDeeplink: Uri? = null
    private var deeplinkListeners: MutableMap<UUID, (Uri) -> Unit> = HashMap()

    fun provideIntent(intent: Intent) {
        Airbridge.getDeeplink(intent, object: AirbridgeCallback<Uri> {
            override fun onSuccess(result: Uri) {
                trackDeeplink(result)
            }

            override fun onFailure(throwable: Throwable) {}
            override fun onComplete() {}
        })
    }

    private fun trackDeeplink(deeplink: Uri) {
        if (startDeeplink == null) {
            startDeeplink = deeplink
        }

        deeplinkListeners.values.forEach { it(deeplink) }
    }

    fun listenDeeplink(listener: (Uri) -> Unit): UUID {
        if (deeplinkListeners.count() == 0) {
            startDeeplink?.let { listener(it) }
        }

        val id = UUID.randomUUID()
        deeplinkListeners[id] = listener

        return id
    }

    fun stopListenDeeplink(id: UUID) {
        deeplinkListeners.remove(id)
    }
}

fun DeeplinkAPI.Companion.fromPlugin(binding: FlutterPlugin.FlutterPluginBinding): DeeplinkAPI {
    return DeeplinkAPI(EventChannel(binding.binaryMessenger, "airbridge_flutter_sdk/event/deeplink"))
}