package co.ab180.airbridge.flutter

import co.ab180.airbridge.Airbridge
import co.ab180.airbridge.flutter.utility.trackEvent
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel

class EventAPI(private val channel: MethodChannel) : MethodChannel.MethodCallHandler {
    companion object

    fun attachToChannel() {
        channel.setMethodCallHandler(this)
    }

    fun detachFromChannel() {
        channel.setMethodCallHandler(null)
    }

    override fun onMethodCall(call: MethodCall, result: MethodChannel.Result) {
        val arguments = call.arguments as? Map<*, *> ?: HashMap<Any, Any>()

        when (call.method) {
            "trackEvent" -> trackEvent(arguments, result)
            else -> result.notImplemented()
        }
    }

    private fun trackEvent(arguments: Map<*, *>, result: MethodChannel.Result) {
        val category = arguments["category"] as? String
        if (category == null) {
            result.error("BAD_ARGS", "category not found", null)
            return;
        }

        val option = arguments["option"] as? Map<*, *>
        Airbridge.trackEvent(category = category, option = option)

        result.success(null)
    }
}

fun EventAPI.Companion.fromPlugin(binding: FlutterPlugin.FlutterPluginBinding): EventAPI {
    return EventAPI(MethodChannel(binding.binaryMessenger, "airbridge_flutter_sdk/method/event"))
}