//
//  StateAPI.swift
//  airbridge_flutter_sdk
//
//  Created by WOF on 2021/07/05.
//

import Foundation
import AirBridge

struct StateAPI {
    let channel: FlutterMethodChannel
}

extension StateAPI {
    mutating func attachToChannel() {
        channel.setMethodCallHandler(handle(call:result:))
    }
    
    mutating func detachFromChannel() {
        channel.setMethodCallHandler(nil)
    }
}

extension StateAPI {
    func handle(call: FlutterMethodCall, result: FlutterResult) {
        let arguments = call.arguments as? [AnyHashable: Any] ?? [:]
        
        switch call.method {
        case "setUser": setUser(arguments, result)
        case "updateUser": updateUser(arguments, result)
        case "startTracking": startTracking(arguments, result)
        case "getDeviceUUID": getDeviceUUID(arguments, result)
        case "setDeviceAlias": setDeviceAlias(arguments, result)
        case "removeDeviceAlias": removeDeviceAlias(arguments, result)
        case "clearDeviceAlias": clearDeviceAlias(arguments, result)
        case "registerPushToken": registerPushToken(arguments, result)
        default: result(FlutterMethodNotImplemented)
        }
    }
}

extension StateAPI {
    func setUser(_ arguments: [AnyHashable: Any], _ result: FlutterResult) {
        AirBridge.state().setUser(from: arguments)
        
        result(nil)
    }
    
    func updateUser(_ arguments: [AnyHashable: Any], _ result: FlutterResult) {
        AirBridge.state().updateUser(from: arguments)
        
        result(nil)
    }
    
    func startTracking(_ arguments: [AnyHashable: Any], _ result: FlutterResult) {
        AirBridge.startTracking()
        
        result(nil)
    }
    
    func getDeviceUUID(_ arguments: [AnyHashable: Any], _ result: FlutterResult) {
        result(AirBridge.deviceUUID())
    }

    func setDeviceAlias(_ arguments: [AnyHashable: Any], _ result: FlutterResult) {
        guard let key = arguments["key"] as? String else {
            result(FlutterError.init(code: "BAD_ARGS",
                                         message: "key not found",
                                         details: nil))
            return
        }

        guard let value = arguments["value"] as? String else {
            result(FlutterError.init(code: "BAD_ARGS",
                                         message: "value not found",
                                         details: nil))
            return
        }

        AirBridge.state().setDeviceAlias(key: key, value: value)
        
        result(nil)
    }

    func removeDeviceAlias(_ arguments: [AnyHashable: Any], _ result: FlutterResult) {
         guard let key = arguments["key"] as? String else {
            result(FlutterError.init(code: "BAD_ARGS",
                                         message: "key not found",
                                         details: nil))
            return
        }

        AirBridge.state().removeDeviceAlias(key: key)
        
        result(nil)
    }

    func clearDeviceAlias(_ arguments: [AnyHashable: Any], _ result: FlutterResult) {
        AirBridge.state().clearDeviceAlias()
        
        result(nil)
    }

    func registerPushToken(_ arguments: [AnyHashable: Any], _ result: FlutterResult) {
         guard let token = arguments["token"] as? String else {
            result(FlutterError.init(code: "BAD_ARGS",
                                         message: "token not found",
                                         details: nil))
            return
        }

        let data = token.data(using: .utf8)
        AirBridge.registerPushToken(data)
        
        result(nil)
    }
}

extension StateAPI {
    static func fromPlugin(_ registrar: FlutterPluginRegistrar) -> StateAPI {
        return StateAPI(channel: FlutterMethodChannel(name: "airbridge_flutter_sdk/method/state",
                                                      binaryMessenger: registrar.messenger()))
    }
}

