//
//  Airbridge.swift
//  airbridge_flutter_sdk
//
//  Created by WOF on 2021/07/05.
//

import Foundation
import AirBridge

extension AirBridge {
    static func setConfigFromAirbridgeJSON() {
        guard let dictionary = FlutterDartProject.loadDictinoaryAsset(name: "assets/airbridge.json") else { return }
        
        if let sessionTimeoutSeconds = dictionary["sessionTimeoutSeconds"] as? Int {
            AirBridge.setSessionTimeout(sessionTimeoutSeconds * 1000)
        }
        if let autoStartTrackingEnabled = dictionary["autoStartTrackingEnabled"] as? Bool {
            AirBridge.setAutoStartTrackingEnabled(autoStartTrackingEnabled)
        }
        if let userInfoHashEnabled = dictionary["userInfoHashEnabled"] as? Bool {
            AirBridge.setIsUserInfoHashed(userInfoHashEnabled)
        }
        if let trackAirbridgeLinkOnly = dictionary["trackAirbridgeLinkOnly"] as? Bool {
            AirBridge.setIsTrackAirbridgeDeeplinkOnly(trackAirbridgeLinkOnly)
        }
        if let facebookDeferredAppLinkEnabled = dictionary["facebookDeferredAppLinkEnabled"] as? Bool {
            AirBridge.setIsFacebookDeferredAppLinkEnabled(facebookDeferredAppLinkEnabled)
        }
        if let trackingAuthorizeTimeoutSeconds = dictionary["trackingAuthorizeTimeoutSeconds"] as? UInt64 {
            AirBridge.setting().trackingAuthorizeTimeout = trackingAuthorizeTimeoutSeconds * 1000
        }
        if 
            let secretID = dictionary["sdkSignatureSecretID"] as? String,
            let secretKey = dictionary["sdkSignatureSecret"] as? String 
        {
            AirBridge.setSDKSignatureSecret(
                id: secretID,
                secret: secretKey
            )
        }

        let logLevel = dictionary["logLevel"] as? String ?? ""
        AirBridge.setLogLevel([
            "debug": ABLogLevel.LOG_DEBUG,
            "info": ABLogLevel.LOG_INFO,
            "warning": ABLogLevel.LOG_WARNING,
            "error": ABLogLevel.LOG_CRITICAL,
            "fault": ABLogLevel.LOG_CRASH
        ][logLevel.lowercased()] ?? ABLogLevel.LOG_WARNING)
    }
}

extension AirBridge {
    func trackEvent(category: String, option: [AnyHashable: Any]?) {
        let event = ABInAppEvent()
        
        event?.setCategory(category)
        
        if let option = option {
            event?.setAction(option["action"] as? String)
            event?.setLabel(option["label"] as? String)
            event?.setValue(option["value"] as? NSNumber)
            event?.setCustoms((option["customAttributes"] as? [AnyHashable: Any])?.toCustomsDictionary())
            event?.setSemantics((option["semanticAttributes"] as? [AnyHashable: Any])?.toSemanticsDictionary())
        }
        
        event?.send()
    }
}

fileprivate extension Dictionary where Key == AnyHashable, Value == Any {
    func toCustomsDictionary() -> [String: Any] {
        var dictionary: [String: Any] = [:]
        
        forEach { key, value in
            if let key = key as? String {
                dictionary[key] = value
            }
        }
        
        return dictionary
    }
    
    func toSemanticsDictionary() -> [String: Any] {
        var dictionary: [String: Any] = [:]
        
        forEach { key, value in
            if let key = key as? String {
                dictionary[key] = value
            }
        }
        
        return dictionary
    }
}
